import pytest
from pydantic_forms.exceptions import FormValidationError

from gso.products.product_types.site import Site
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.mark.workflow()
def test_modify_site(responses, site_subscription_factory, faker):
    subscription_id = site_subscription_factory()
    initial_site_data = [
        {"subscription_id": subscription_id},
        {
            "site_bgp_community_id": faker.pyint(),
            "site_internal_id": faker.pyint(),
            "site_ts_address": faker.ipv4(),
        },
        {},
    ]
    result, _, _ = run_workflow("modify_site", initial_site_data)
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Site.from_subscription(subscription_id)
    assert subscription.status == "active"
    assert subscription.site.site_bgp_community_id == initial_site_data[1]["site_bgp_community_id"]
    assert subscription.site.site_internal_id == initial_site_data[1]["site_internal_id"]


@pytest.mark.workflow()
def test_modify_site_with_duplicate_bgp_community_id(faker, site_subscription_factory):
    duplicate_bgp_community_id = faker.pyint()

    site_subscription_factory(site_bgp_community_id=duplicate_bgp_community_id)
    subscription_b = site_subscription_factory()

    initial_site_data = [
        {"subscription_id": subscription_b},
        {
            "site_bgp_community_id": duplicate_bgp_community_id,
        },
        {},
    ]

    with pytest.raises(FormValidationError, match="site_bgp_community_id must be unique"):
        run_workflow("modify_site", initial_site_data)


@pytest.mark.workflow()
def test_modify_site_with_duplicate_internal_id(faker, site_subscription_factory):
    duplicate_internal_id = faker.pyint()

    site_subscription_factory(site_internal_id=duplicate_internal_id)
    subscription_b = site_subscription_factory()

    initial_site_data = [
        {"subscription_id": subscription_b},
        {
            "site_internal_id": duplicate_internal_id,
        },
        {},
    ]

    with pytest.raises(FormValidationError, match="site_internal_id must be unique"):
        run_workflow("modify_site", initial_site_data)


@pytest.mark.workflow()
def test_modify_site_with_duplicate_ts_address(faker, site_subscription_factory):
    duplicate_ts_address = faker.ipv4()

    site_subscription_factory(site_ts_address=duplicate_ts_address)
    subscription_b = site_subscription_factory()

    initial_site_data = [
        {"subscription_id": subscription_b},
        {
            "site_ts_address": duplicate_ts_address,
        },
        {},
    ]

    with pytest.raises(FormValidationError, match="site_ts_address must be unique"):
        run_workflow("modify_site", initial_site_data)
